@@ -1,3 +1,4 @@
+use std::fmt::Display;
 use std::net::IpAddr;
 use std::path::PathBuf;
 
@@ -15,6 +16,21 @@ pub enum MediaType {
     Video,
 }
 
+#[derive(ValueEnum, Clone)]
+pub enum SizeDisplay {
+    Human,
+    Exact,
+}
+
+impl Display for SizeDisplay {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        match self {
+            SizeDisplay::Human => write!(f, "human"),
+            SizeDisplay::Exact => write!(f, "exact"),
+        }
+    }
+}
+
 #[derive(Parser)]
 #[command(name = "miniserve", author, about, version)]
 pub struct CliArgs {
@@ -26,6 +42,21 @@ pub struct CliArgs {
     #[arg(value_hint = ValueHint::AnyPath, env = "MINISERVE_PATH")]
     pub path: Option<PathBuf>,
 
+    /// The path to where file uploads will be written to before being moved to their
+    /// correct location. It's wise to make sure that this directory will be written to
+    /// disk and not into memory.
+    ///
+    /// This value will only be used **IF** file uploading is enabled. If this option is
+    /// not set, the operating system default temporary directory will be used.
+    #[arg(
+        long = "temp-directory",
+        value_hint = ValueHint::FilePath,
+        requires = "allowed_upload_dir",
+        value_parser(validate_is_dir_and_exists),
+        env = "MINISERVER_TEMP_UPLOAD_DIRECTORY")
+    ]
+    pub temp_upload_directory: Option<PathBuf>,
+
     /// The name of a directory index file to serve, like "index.html"
     ///
     /// Normally, when miniserve serves a directory, it creates a listing for that directory.
@@ -166,6 +197,32 @@ pub struct CliArgs {
     #[arg(short = 'u', long = "upload-files", value_hint = ValueHint::FilePath, num_args(0..=1), value_delimiter(','), env = "MINISERVE_ALLOWED_UPLOAD_DIR")]
     pub allowed_upload_dir: Option<Vec<PathBuf>>,
 
+    /// Configure amount of concurrent uploads when visiting the website. Must have
+    /// upload-files option enabled for this setting to matter.
+    ///
+    /// For example, a value of 4 would mean that the web browser will only upload
+    /// 4 files at a time to the web server when using the web browser interface.
+    ///
+    /// When the value is kept at 0, it attempts to resolve all the uploads at once
+    /// in the web browser.
+    ///
+    /// NOTE: Web pages have a limit of how many active HTTP connections that they
+    /// can make at one time, so even though you might set a concurrency limit of
+    /// 100, the browser might only make progress on the max amount of connections
+    /// it allows the web page to have open.
+    #[arg(
+        long = "web-upload-files-concurrency",
+        env = "MINISERVE_WEB_UPLOAD_CONCURRENCY",
+        default_value = "0"
+    )]
+    pub web_upload_concurrency: usize,
+
+    /// Enable recursive directory size calculation
+    ///
+    /// This is disabled by default because it is a potentially fairly IO intensive operation.
+    #[arg(long = "directory-size", env = "MINISERVE_DIRECTORY_SIZE")]
+    pub directory_size: bool,
+
     /// Enable creating directories
     #[arg(
         short = 'U',
@@ -315,6 +372,21 @@ pub struct CliArgs {
     /// Currently incompatible with -P|--no-symlinks (see https://github.com/messense/dav-server-rs/issues/37)
     #[arg(long, env = "MINISERVE_ENABLE_WEBDAV", conflicts_with = "no_symlinks")]
     pub enable_webdav: bool,
+
+    /// Show served file size in exact bytes
+    #[arg(long, default_value_t = SizeDisplay::Human, env = "MINISERVE_SIZE_DISPLAY")]
+    pub size_display: SizeDisplay,
+
+    /// Optional external URL (e.g., 'http://external.example.com:8081') prepended to file links in listings.
+    ///
+    /// Allows serving files from a different URL than the browsing instance. Useful for setups like:
+    /// one authenticated instance for browsing, linking files (via this option) to a second,
+    /// non-indexed (-I) instance for direct downloads. This obscures the full file list on
+    /// the download server, while users can still copy direct file URLs for sharing.
+    /// The external URL is put verbatim in front of the relative location of the file, including the protocol.
+    /// The user should take care this results in a valid URL, no further checks are being done.
+    #[arg(long = "file-external-url", env = "MINISERVE_FILE_EXTERNAL_URL")]
+    pub file_external_url: Option<String>,
 }
 
 /// Checks whether an interface is valid, i.e. it can be parsed into an IP address
@@ -322,10 +394,26 @@ fn parse_interface(src: &str) -> Result<IpAddr, std::net::AddrParseError> {
     src.parse::<IpAddr>()
 }
 
+/// Validate that a path passed in is a directory and it exists.
+fn validate_is_dir_and_exists(s: &str) -> Result<PathBuf, String> {
+    let path = PathBuf::from(s);
+    if path.exists() && path.is_dir() {
+        Ok(path)
+    } else {
+        Err(format!(
+            "Upload temporary directory must exist and be a directory. \
+            Validate that path {:?} meets those requirements.",
+            path
+        ))
+    }
+}
+
 #[derive(Clone, Debug, thiserror::Error)]
 pub enum AuthParseError {
     /// Might occur if the HTTP credential string does not respect the expected format
-    #[error("Invalid format for credentials string. Expected username:password, username:sha256:hash or username:sha512:hash")]
+    #[error(
+        "Invalid format for credentials string. Expected username:password, username:sha256:hash or username:sha512:hash"
+    )]
     InvalidAuthFormat,
 
     /// Might occur if the hash method is neither sha256 nor sha512
