@@ -6,13 +6,13 @@ use std::{
 };
 
 use actix_web::http::header::HeaderMap;
-use anyhow::{anyhow, Context, Result};
+use anyhow::{Context, Result, anyhow};
 
 #[cfg(feature = "tls")]
 use rustls_pemfile as pemfile;
 
 use crate::{
-    args::{parse_auth, CliArgs, MediaType},
+    args::{CliArgs, MediaType, parse_auth},
     auth::RequiredAuth,
     file_utils::sanitize_path,
     listing::{SortingMethod, SortingOrder},
@@ -24,7 +24,7 @@ const ROUTE_ALPHABET: [char; 16] = [
     '1', '2', '3', '4', '5', '6', '7', '8', '9', '0', 'a', 'b', 'c', 'd', 'e', 'f',
 ];
 
-#[derive(Clone)]
+#[derive(Debug, Clone)]
 /// Configuration of the Miniserve application
 pub struct MiniserveConfig {
     /// Enable verbose mode
@@ -33,6 +33,9 @@ pub struct MiniserveConfig {
     /// Path to be served by miniserve
     pub path: std::path::PathBuf,
 
+    /// Temporary directory that should be used when files are uploaded to the server
+    pub temp_upload_directory: Option<std::path::PathBuf>,
+
     /// Port on which miniserve will be listening
     pub port: u16,
 
@@ -60,10 +63,16 @@ pub struct MiniserveConfig {
     /// Route prefix; Either empty or prefixed with slash
     pub route_prefix: String,
 
-    /// Randomly generated favicon route
+    /// Well-known healthcheck route (prefixed if route_prefix is provided)
+    pub healthcheck_route: String,
+
+    /// Well-known API route (prefixed if route_prefix is provided)
+    pub api_route: String,
+
+    /// Well-known favicon route (prefixed if route_prefix is provided)
     pub favicon_route: String,
 
-    /// Randomly generated css route
+    /// Well-known css route (prefixed if route_prefix is provided)
     pub css_route: String,
 
     /// Default color scheme
@@ -95,12 +104,18 @@ pub struct MiniserveConfig {
     /// Enable QR code display
     pub show_qrcode: bool,
 
+    /// Enable recursive directory size calculation
+    pub directory_size: bool,
+
     /// Enable creating directories
     pub mkdir_enabled: bool,
 
     /// Enable file upload
     pub file_upload: bool,
 
+    /// Max amount of concurrency when uploading multiple files
+    pub web_upload_concurrency: usize,
+
     /// List of allowed upload directories
     pub allowed_upload_dir: Vec<String>,
 
@@ -152,12 +167,18 @@ pub struct MiniserveConfig {
     /// If enabled, respond to WebDAV requests (read-only).
     pub webdav_enabled: bool,
 
+    /// If enabled, will show in exact byte size of the file
+    pub show_exact_bytes: bool,
+
     /// If set, use provided rustls config for TLS
     #[cfg(feature = "tls")]
     pub tls_rustls_config: Option<rustls::ServerConfig>,
 
     #[cfg(not(feature = "tls"))]
     pub tls_rustls_config: Option<()>,
+
+    /// Optional external URL to prepend to file links in listings
+    pub file_external_url: Option<String>,
 }
 
 impl MiniserveConfig {
@@ -189,19 +210,23 @@ impl MiniserveConfig {
             }
         }
 
-        // Generate some random routes for the favicon and css so that they are very unlikely to conflict with
-        // real files.
-        // If --random-route is enabled , in order to not leak the random generated route, we must not use it
+        // Format some well-known routes at paths that are very unlikely to conflict with real
+        // files.
+        // If --random-route is enabled, in order to not leak the random generated route, we must not use it
         // as static files prefix.
         // Otherwise, we should apply route_prefix to static files.
-        let (favicon_route, css_route) = if args.random_route {
+        let (healthcheck_route, api_route, favicon_route, css_route) = if args.random_route {
             (
+                "/__miniserve_internal/healthcheck".into(),
+                "/__miniserve_internal/api".into(),
                 "/__miniserve_internal/favicon.svg".into(),
                 "/__miniserve_internal/style.css".into(),
             )
         } else {
             (
-                format!("{}/{}", route_prefix, "__miniserve_internal/favicon.ico"),
+                format!("{}/{}", route_prefix, "__miniserve_internal/healthcheck"),
+                format!("{}/{}", route_prefix, "__miniserve_internal/api"),
+                format!("{}/{}", route_prefix, "__miniserve_internal/favicon.svg"),
                 format!("{}/{}", route_prefix, "__miniserve_internal/style.css"),
             )
         };
@@ -272,9 +297,15 @@ impl MiniserveConfig {
             .transpose()?
             .unwrap_or_default();
 
+        let show_exact_bytes = match args.size_display {
+            crate::args::SizeDisplay::Human => false,
+            crate::args::SizeDisplay::Exact => true,
+        };
+
         Ok(Self {
             verbose: args.verbose,
             path: args.path.unwrap_or_else(|| PathBuf::from(".")),
+            temp_upload_directory: args.temp_upload_directory,
             port,
             interfaces,
             auth,
@@ -284,6 +315,8 @@ impl MiniserveConfig {
             default_sorting_method: args.default_sorting_method,
             default_sorting_order: args.default_sorting_order,
             route_prefix,
+            healthcheck_route,
+            api_route,
             favicon_route,
             css_route,
             default_color_scheme,
@@ -293,8 +326,10 @@ impl MiniserveConfig {
             pretty_urls: args.pretty_urls,
             overwrite_files: args.overwrite_files,
             show_qrcode: args.qrcode,
+            directory_size: args.directory_size,
             mkdir_enabled: args.mkdir_enabled,
             file_upload: args.allowed_upload_dir.is_some(),
+            web_upload_concurrency: args.web_upload_concurrency,
             allowed_upload_dir,
             uploadable_media_type,
             tar_enabled: args.enable_tar,
@@ -312,6 +347,8 @@ impl MiniserveConfig {
             webdav_enabled: args.enable_webdav,
             tls_rustls_config: tls_rustls_server_config,
             compress_response: args.compress_response,
+            show_exact_bytes,
+            file_external_url: args.file_external_url,
         })
     }
 }
