@@ -13,8 +13,9 @@ use select::{document::Document, node::Node, predicate::Attr};
 mod fixtures;
 
 use crate::fixtures::{
-    port, server, tmpdir, Error, TestServer, DIRECTORIES, DIRECTORY_SYMLINK, FILES, FILE_SYMLINK,
-    HIDDEN_DIRECTORIES, HIDDEN_FILES,
+    DIR_BEHIND_SYMLINKED_DIR, DIRECTORIES, DIRECTORY_SYMLINK, Error,
+    FILE_IN_DIR_BEHIND_SYMLINKED_DIR, FILE_SYMLINK, FILES, HIDDEN_DIRECTORIES, HIDDEN_FILES,
+    TestServer, port, server, tmpdir,
 };
 
 #[rstest]
@@ -55,23 +56,49 @@ fn serves_requests_with_non_default_port(server: TestServer) -> Result<(), Error
     }
 
     for &directory in DIRECTORIES {
-        assert!(parsed
-            .find(|x: &Node| x.text() == directory)
-            .next()
-            .is_some());
+        assert!(
+            parsed
+                .find(|x: &Node| x.text() == directory)
+                .next()
+                .is_some()
+        );
         let dir_body = reqwest::blocking::get(server.url().join(directory)?)?.error_for_status()?;
         let dir_body_parsed = Document::from_read(dir_body)?;
         for &file in FILES {
-            assert!(dir_body_parsed
-                .find(|x: &Node| x.text() == file)
-                .next()
-                .is_some());
+            assert!(
+                dir_body_parsed
+                    .find(|x: &Node| x.text() == file)
+                    .next()
+                    .is_some()
+            );
         }
     }
 
     Ok(())
 }
 
+#[rstest]
+#[case("__miniserve_internal/healthcheck", server(None::<&str>))]
+#[case("__miniserve_internal/favicon.svg", server(None::<&str>))]
+#[case("__miniserve_internal/style.css", server(None::<&str>))]
+#[case("testlol/__miniserve_internal/healthcheck", server(&["--route-prefix", "testlol"]))]
+#[case("testlol/__miniserve_internal/favicon.svg", server(&["--route-prefix", "testlol"]))]
+#[case("testlol/__miniserve_internal/style.css", server(&["--route-prefix", "testlol"]))]
+#[case("__miniserve_internal/healthcheck", server(&["--random-route"]))]
+#[case("__miniserve_internal/favicon.svg", server(&["--random-route"]))]
+#[case("__miniserve_internal/style.css", server(&["--random-route"]))]
+#[case("__miniserve_internal/healthcheck", server(&["--auth", "doesnt:matter"]))]
+#[case("__miniserve_internal/favicon.svg", server(&["--auth", "doesnt:matter"]))]
+#[case("__miniserve_internal/style.css", server(&["--auth", "doesnt:matter"]))]
+fn serves_requests_for_special_routes(
+    #[case] route: &str,
+    #[case] server: TestServer,
+) -> Result<(), Error> {
+    reqwest::blocking::get(format!("{}{}", server.url(), route))?.error_for_status()?;
+
+    Ok(())
+}
+
 #[rstest]
 fn serves_requests_hidden_files(#[with(&["--hidden"])] server: TestServer) -> Result<(), Error> {
     let body = reqwest::blocking::get(server.url())?.error_for_status()?;
@@ -86,17 +113,21 @@ fn serves_requests_hidden_files(#[with(&["--hidden"])] server: TestServer) -> Re
     }
 
     for &directory in DIRECTORIES.iter().chain(HIDDEN_DIRECTORIES) {
-        assert!(parsed
-            .find(|x: &Node| x.text() == directory)
-            .next()
-            .is_some());
+        assert!(
+            parsed
+                .find(|x: &Node| x.text() == directory)
+                .next()
+                .is_some()
+        );
         let dir_body = reqwest::blocking::get(server.url().join(directory)?)?.error_for_status()?;
         let dir_body_parsed = Document::from_read(dir_body)?;
         for &file in FILES.iter().chain(HIDDEN_FILES) {
-            assert!(dir_body_parsed
-                .find(|x: &Node| x.text() == file)
-                .next()
-                .is_some());
+            assert!(
+                dir_body_parsed
+                    .find(|x: &Node| x.text() == file)
+                    .next()
+                    .is_some()
+            );
         }
     }
 
@@ -109,10 +140,12 @@ fn serves_requests_no_hidden_files_without_flag(server: TestServer) -> Result<()
     let parsed = Document::from_read(body)?;
 
     for &hidden_item in HIDDEN_FILES.iter().chain(HIDDEN_DIRECTORIES) {
-        assert!(parsed
-            .find(|x: &Node| x.text() == hidden_item)
-            .next()
-            .is_none());
+        assert!(
+            parsed
+                .find(|x: &Node| x.text() == hidden_item)
+                .next()
+                .is_none()
+        );
         let resp = reqwest::blocking::get(server.url().join(hidden_item)?)?;
         assert_eq!(resp.status(), StatusCode::BAD_REQUEST);
     }
@@ -120,6 +153,27 @@ fn serves_requests_no_hidden_files_without_flag(server: TestServer) -> Result<()
     Ok(())
 }
 
+#[rstest]
+#[case(server(None::<&str>), StatusCode::OK)]
+#[case(server(&["--no-symlinks"]), StatusCode::NOT_FOUND)]
+fn serves_requests_nested_in_symlinks(
+    #[case] server: TestServer,
+    #[case] expected_status: StatusCode,
+) -> Result<(), Error> {
+    let file_status =
+        reqwest::blocking::get(server.url().join(DIRECTORY_SYMLINK)?.join(FILES[0])?)?.status();
+    assert_eq!(file_status, expected_status);
+
+    let dir_status = reqwest::blocking::get(server.url().join(DIR_BEHIND_SYMLINKED_DIR)?)?.status();
+    assert_eq!(dir_status, expected_status);
+
+    let nested_file_status =
+        reqwest::blocking::get(server.url().join(FILE_IN_DIR_BEHIND_SYMLINKED_DIR)?)?.status();
+    assert_eq!(nested_file_status, expected_status);
+
+    Ok(())
+}
+
 #[rstest]
 #[case(true, false, server(&["--no-symlinks"]))]
 #[case(true, true, server(&["--no-symlinks", "--show-symlink-info"]))]
@@ -172,10 +226,12 @@ fn serves_requests_symlinks(
             assert_eq!(node.unwrap().attr("class").unwrap(), "file");
         }
     }
-    assert!(parsed
-        .find(|x: &Node| x.text() == BROKEN_SYMLINK)
-        .next()
-        .is_none());
+    assert!(
+        parsed
+            .find(|x: &Node| x.text() == BROKEN_SYMLINK)
+            .next()
+            .is_none()
+    );
 
     Ok(())
 }
@@ -248,10 +304,12 @@ fn serve_index_instead_of_404_in_spa_mode(
 ) -> Result<(), Error> {
     let body = reqwest::blocking::get(format!("{}{}", server.url(), url))?.error_for_status()?;
     let parsed = Document::from_read(body)?;
-    assert!(parsed
-        .find(|x: &Node| x.text() == "Test Hello Yes")
-        .next()
-        .is_some());
+    assert!(
+        parsed
+            .find(|x: &Node| x.text() == "Test Hello Yes")
+            .next()
+            .is_some()
+    );
 
     Ok(())
 }
@@ -266,10 +324,12 @@ fn serve_file_instead_of_404_in_pretty_urls_mode(
 ) -> Result<(), Error> {
     let body = reqwest::blocking::get(format!("{}{}", server.url(), url))?.error_for_status()?;
     let parsed = Document::from_read(body)?;
-    assert!(parsed
-        .find(|x: &Node| x.text() == "Test Hello Yes")
-        .next()
-        .is_some());
+    assert!(
+        parsed
+            .find(|x: &Node| x.text() == "Test Hello Yes")
+            .next()
+            .is_some()
+    );
 
     Ok(())
 }
@@ -301,12 +361,16 @@ fn serves_requests_static_file_check(
     let parsed = Document::from_read(body)?;
     let re = Regex::new(&static_file_pattern).unwrap();
 
-    assert!(parsed
-        .find(Attr("rel", "stylesheet"))
-        .all(|x| re.is_match(x.attr("href").unwrap())));
-    assert!(parsed
-        .find(Attr("rel", "icon"))
-        .all(|x| re.is_match(x.attr("href").unwrap())));
+    assert!(
+        parsed
+            .find(Attr("rel", "stylesheet"))
+            .all(|x| re.is_match(x.attr("href").unwrap()))
+    );
+    assert!(
+        parsed
+            .find(Attr("rel", "icon"))
+            .all(|x| re.is_match(x.attr("href").unwrap()))
+    );
 
     Ok(())
 }
@@ -318,22 +382,30 @@ fn serves_no_directory_if_indexing_disabled(#[case] server: TestServer) -> Resul
     assert_eq!(body.status(), StatusCode::NOT_FOUND);
     let parsed = Document::from_read(body)?;
 
-    assert!(parsed
-        .find(|x: &Node| x.text() == FILES[0])
-        .next()
-        .is_none());
-    assert!(parsed
-        .find(|x: &Node| x.text() == DIRECTORIES[0])
-        .next()
-        .is_none());
-    assert!(parsed
-        .find(|x: &Node| x.text() == "404 Not Found")
-        .next()
-        .is_some());
-    assert!(parsed
-        .find(|x: &Node| x.text() == "File not found.")
-        .next()
-        .is_some());
+    assert!(
+        parsed
+            .find(|x: &Node| x.text() == FILES[0])
+            .next()
+            .is_none()
+    );
+    assert!(
+        parsed
+            .find(|x: &Node| x.text() == DIRECTORIES[0])
+            .next()
+            .is_none()
+    );
+    assert!(
+        parsed
+            .find(|x: &Node| x.text() == "404 Not Found")
+            .next()
+            .is_some()
+    );
+    assert!(
+        parsed
+            .find(|x: &Node| x.text() == "File not found.")
+            .next()
+            .is_some()
+    );
 
     Ok(())
 }
